@@ -28,6 +28,7 @@ require 'uri'
 require 'webrick/httputils'
 require 'zlib'
 require 'stringio'
+require 'digest/md5'
 require 'mechanize/monkey_patch'
 require 'mechanize/cookie'
 require 'mechanize/errors'
@@ -63,7 +64,7 @@ class Mechanize
   ##
   # The version of Mechanize you are using.
  
-  VERSION = '0.6.8'
+  VERSION = '0.6.11'
 
   ##
   # User Agent aliases
@@ -92,12 +93,16 @@ class Mechanize
   attr_accessor :keep_alive_time
   attr_accessor :keep_alive
   attr_accessor :conditional_requests
+  attr_accessor :follow_meta_refresh
 
   attr_reader :history
   attr_reader :pluggable_parser
 
   alias :follow_redirect? :redirect_ok
 
+  @@nonce_count = -1
+  CNONCE = Digest::MD5.hexdigest("%x" % (Time.now.to_i + rand(65535)))
+
   def initialize
     # attr_accessors
     @cookie_jar     = CookieJar.new
@@ -116,9 +121,11 @@ class Mechanize
     @history        = WWW::Mechanize::History.new
     @pluggable_parser = PluggableParser.new
 
-    # Basic Auth variables
-    @user           = nil # Basic Auth User
-    @password       = nil # Basic Auth Password
+    # Auth variables
+    @user           = nil # Auth User
+    @password       = nil # Auth Password
+    @digest         = nil # DigestAuth Digest
+    @auth_hash      = {}  # Keep track of urls for sending auth
 
     # Proxy settings
     @proxy_addr     = nil
@@ -128,6 +135,8 @@ class Mechanize
 
     @conditional_requests = true
 
+    @follow_meta_refresh  = false
+
     # Connection Cache & Keep alive
     @connection_cache = {}
     @keep_alive_time  = 300
@@ -157,8 +166,12 @@ class Mechanize
 
   # Sets the user and password to be used for basic authentication.
   def basic_auth(user, password)
-    @user = user
-    @password = password
+    auth(user, password)
+  end
+
+  def auth(user, password)
+    @user       = user
+    @password   = password
   end
 
   # Fetches the URL passed in and returns a page.
@@ -228,7 +241,7 @@ class Mechanize
   #  agent.submit(page.forms.first, page.forms.first.buttons.first)
   def submit(form, button=nil)
     form.add_button_to_query(button) if button
-    uri = to_absolute_uri(form.action)
+    uri = to_absolute_uri(form.action, form.page)
     case form.method.upcase
     when 'POST'
       post_form(uri, form) 
@@ -309,29 +322,87 @@ class Mechanize
       end
     end
 
-    request.basic_auth(@user, @password) if @user || @password
+    if( @auth_hash[uri.host] )
+      case @auth_hash[uri.host]
+      when :basic
+        request.basic_auth(@user, @password)
+      when :digest
+        @digest_response ||= nil
+        @digest_response = self.gen_auth_header(uri,request,@digest) if @digest
+        request.add_field('Authorization', @digest_response) if @digest_response
+      end
+    end
+
     request
   end
 
+  def gen_auth_header(uri, request, auth_header, is_IIS = false)
+    @@nonce_count += 1
+
+    user = @digest_user
+    password = @digest_password
+
+    auth_header =~ /^(\w+) (.*)/
+
+    params = {}
+    $2.gsub(/(\w+)="(.*?)"/) { params[$1] = $2 }
+
+    a_1 = "#{@user}:#{params['realm']}:#{@password}"
+    a_2 = "#{request.method}:#{uri.path}"
+    request_digest = ''
+    request_digest << Digest::MD5.hexdigest(a_1)
+    request_digest << ':' << params['nonce']
+    request_digest << ':' << ('%08x' % @@nonce_count)
+    request_digest << ':' << CNONCE
+    request_digest << ':' << params['qop']
+    request_digest << ':' << Digest::MD5.hexdigest(a_2)
+
+    header = ''
+    header << "Digest username=\"#{@user}\", "
+    header << "realm=\"#{params['realm']}\", "
+    if is_IIS then
+      header << "qop=\"#{params['qop']}\", "
+    else
+      header << "qop=#{params['qop']}, "
+    end
+    header << "uri=\"#{uri.path}\", "
+    header << "algorithm=MD5, "
+    header << "nonce=\"#{params['nonce']}\", "
+    header << "nc=#{'%08x' % @@nonce_count}, "
+    header << "cnonce=\"#{CNONCE}\", "
+    header << "response=\"#{Digest::MD5.hexdigest(request_digest)}\""
+
+    return header
+  end
+
   private
 
   def to_absolute_uri(url, cur_page=current_page())
     unless url.is_a? URI
-      url = url.to_s.strip
+      url = url.to_s.strip.gsub(/[^#{0.chr}-#{125.chr}]/) { |match|
+        sprintf('%%%X', match.unpack($KCODE == 'UTF8' ? 'U' : 'c')[0])
+      }
+
       url = URI.parse(
               Util.html_unescape(
-                url.split(/%[0-9A-Fa-f]{2}/).zip(
-                  url.scan(/%[0-9A-Fa-f]{2}/)
+                url.split(/%[0-9A-Fa-f]{2}|#/).zip(
+                  url.scan(/%[0-9A-Fa-f]{2}|#/)
                 ).map { |x,y|
                   "#{URI.escape(x)}#{y}"
-                }.join('').gsub(/%23/, '#')
+                }.join('')
               )
             )
     end
 
+    url.path = '/' if url.path.length == 0
+
     # construct an absolute uri
     if url.relative?
       raise 'no history. please specify an absolute URL' unless cur_page.uri
+      base = cur_page.respond_to?(:bases) ? cur_page.bases.last : nil
+      url = ((base && base.uri && base.uri.absolute?) ?
+              base.uri :
+              cur_page.uri) + url
       url = cur_page.uri + url
       # Strip initial "/.." bits from the path
       url.path.sub!(/^(\/\.\.)+(?=\/)/, '')
@@ -467,6 +538,8 @@ class Mechanize
         when 'gzip'
           log.debug('gunzip body') if log
           Zlib::GzipReader.new(body).read
+        when 'x-gzip'
+          body.read
         else
           raise 'Unsupported content encoding'
         end
@@ -509,6 +582,11 @@ class Mechanize
 
     res_klass = Net::HTTPResponse::CODE_TO_OBJ[page.code.to_s]
 
+    if follow_meta_refresh && page.respond_to?(:meta) &&
+      (redirect = page.meta.first)
+      return redirect.click
+    end
+
     return page if res_klass <= Net::HTTPSuccess
 
     if res_klass == Net::HTTPNotModified
@@ -522,6 +600,20 @@ class Mechanize
       page = fetch_page(abs_uri, fetch_request(abs_uri), page)
       @history.push(page, from_uri)
       return page
+    elsif res_klass <= Net::HTTPUnauthorized
+      raise ResponseCodeError.new(page) unless @user || @password
+      raise ResponseCodeError.new(page) if @auth_hash.has_key?(uri.host)
+      if response['www-authenticate'] =~ /Digest/i
+        @auth_hash[uri.host] = :digest
+        @digest = response['www-authenticate']
+      else
+        @auth_hash[uri.host] = :basic
+      end
+      return fetch_page(  uri,
+                          fetch_request(uri, request.method.downcase.to_sym),
+                          cur_page,
+                          request_data
+                       )
     end
 
     raise ResponseCodeError.new(page), "Unhandled response", caller
@@ -540,7 +632,7 @@ class Mechanize
   end
 
   def add_to_history(page)
-    @history.push(page)
+    @history.push(page, to_absolute_uri(page.uri))
   end
 
   # :stopdoc:
@@ -555,7 +647,7 @@ class Mechanize
           $1.to_i
         end
 
-        number ? (number.chr rescue match) : match
+        number ? ([number].pack('U') rescue match) : match
       }
     end
   end
