@@ -27,9 +27,11 @@ module WWW
         @response_code = page.code
       end
 
-      def inspect
-        response_code
+      def to_s
+        "#{response_code} => #{Net::HTTPResponse::CODE_TO_OBJ[response_code]}"
       end
+
+      def inspect; to_s; end
     end
   end
 end
