@@ -19,7 +19,7 @@ module WWW
       extend Forwardable
 
       attr_reader :parser, :title, :watch_for_set
-      attr_reader :frames, :iframes, :links, :forms, :meta, :watches
+      attr_reader :frames, :iframes, :links, :forms, :meta, :watches, :bases
       attr_accessor :mech
 
       alias :root :parser
@@ -54,6 +54,7 @@ module WWW
         parse_html if @body && @watch_for_set
       end
 
+      # Find a form with +name+.  Form will be yeilded if a block is given.
       def form(name)
         f = forms.name(name).first
         yield f if block_given?
@@ -68,6 +69,7 @@ module WWW
         @meta     = WWW::Mechanize::List.new
         @frames   = WWW::Mechanize::List.new
         @iframes  = WWW::Mechanize::List.new
+        @bases    = WWW::Mechanize::List.new
         @watches  = {}
     
         # Set the title
@@ -75,6 +77,11 @@ module WWW
           (@parser/'title').text
         end
 
+        # Find all 'base' tags
+        (@parser/'base').each do |node|
+          @bases << Base.new(node, @mech, self)
+        end
+
         # Find all the form tags
         (@parser/'form').each do |html_form|
           form = Form.new(html_form, @mech, self)
@@ -87,6 +94,11 @@ module WWW
           @links << Link.new(node, @mech, self)
         end
 
+        # Find all the 'area' tags
+        (@parser/'area').each do |node|
+          @links << Link.new(node, @mech, self)
+        end
+
         # Find all 'meta' tags
         (@parser/'meta').each do |node|
           next unless node['http-equiv']
@@ -94,7 +106,7 @@ module WWW
           equiv   = node['http-equiv']
           content = node['content']
           if equiv != nil && equiv.downcase == 'refresh'
-            if content != nil && content =~ /^\d+\s*;\s*url\s*=\s*(\S+)/i
+            if content != nil && content =~ /^\d+\s*;\s*url\s*=\s*'?([^\s']+)/i
               node['href'] = $1
               @meta << Meta.new(node, @mech, self)
             end
