@@ -500,6 +500,18 @@ class FormsMechTest < Test::Unit::TestCase
     assert_not_nil(f)
     assert_equal(number_of_fields + 1, form.fields.length)
   end
+  
+  def test_delete_field
+    page = @agent.get("http://localhost:#{PORT}/form_multival.html")
+    form = page.forms.name('post_form').first
+
+    assert_not_nil(form)
+    number_of_fields = form.fields.length
+
+    form.delete_field!('first')
+    assert_nil(form['first'])
+    assert_equal(number_of_fields - 2, form.fields.length)    
+  end
 
   def test_has_field
     page = @agent.get("http://localhost:#{PORT}/form_multival.html")
@@ -511,4 +523,16 @@ class FormsMechTest < Test::Unit::TestCase
     assert_not_nil(f)
     assert_equal(true, form.has_field?('intarweb'))
   end
+
+  def test_field_error
+    @page = @agent.get('http://localhost/empty_form.html')
+    form = @page.forms.first
+    assert_raise(NoMethodError) {
+      form.foo = 'asdfasdf'
+    }
+
+    assert_raise(NoMethodError) {
+      form.foo
+    }
+  end
 end
